/*
 * MP3Info by Thorvald Natvig.
 * Read 'README' and the manpage for details.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include "mp3info.h"
#include <getopt.h>

void printhelp() {
   printf("MP3Info v%i.%i.%i by Thorvald Natvig\n\n",MAJVER,MINVER,PATCHVER);
   printf("Usage: mp3info [options] <filenames>\n\n");
   printf("See manpage for option details.\n");
}

int main (int argc,char *argv[])
{
   Mpfile *file=NULL;
   Tag *newinfo;
   int  opt;
   int  shall_gettag=1, shall_std=1, shall_write=0;
   int  shall_wipe=0, shall_rename=0;
   int  shall_play=0;
   char format[FORMAT_LEN+1];
   char rename[FORMAT_LEN+1];
   char output[STD_BUF_LEN];
   int  errorno=0;
   int  fileno;

   newinfo=new Tag;
   
   strcpy(format,"%IT%It%t%!a, %IE%IE%Ia(%a), %IE%IE%b kbit/s %M %ICCRC %IE%(%mm %ss%)");
   
   while ((opt=getopt(argc,argv,"Ts:f:wWn:t:a:l:y:c:g:G:pN:P"))!=-1) {
      switch(opt) {
       case 'T' :
	 shall_gettag = 0;
	 break ;
       case 's' :
	 shall_std = strtol(optarg,NULL,10);
	 break;
       case 'f' :
	 if (strlen(optarg)>=FORMAT_LEN) fprintf(stderr,"Warning: Format is too long, cut.\n");
	 strncpy(format,optarg,FORMAT_LEN);
	 shall_std = 1;
	 break;
       case 'w' :
	 shall_write = 1;
	 break;
       case 'W' :
	 shall_wipe = 1;
	 shall_std = 0;
	 break;
       case 'n' :
         fprintf(stderr,"Warning: Use -t instead of -n.\n");
	 strncpy(newinfo->title,optarg,30);
	 shall_write = 1;
	 break;
       case 't' :
	 strncpy(newinfo->title,optarg,30);
	 shall_write = 1;
	 break;
       case 'a' :
	 strncpy(newinfo->artist,optarg,30);
	 shall_write = 1;
	 break;
       case 'l' :
	 strncpy(newinfo->album,optarg,30);
	 shall_write = 1;
	 break;
       case 'y' :
	 strncpy(newinfo->year,optarg,4);
	 shall_write = 1;
	 break;
       case 'c' :
	 strncpy(newinfo->comment,optarg,30);
	 shall_write = 1;
	 break;
       case 'g' :
	 newinfo->gennum=strtol(optarg,NULL,10);
	 shall_write = 1;
	 break;
       case 'p' :
         newinfo->printgenres();
         return(0);
       case 'N' :
	 if (strlen(optarg)>=FORMAT_LEN) fprintf(stderr,"Warning: Format is too long, cut.\n");
	 strncpy(rename,optarg,FORMAT_LEN);
	 shall_rename = 1;
	 break;
       case 'P' :
#ifdef PLAYER
         shall_play = 1;
#else
	 printf(stderr,"Warning: No player compiled, not playing.\n");
#endif
         break;
       default :
	 fprintf(stderr,"Sorry, option not implemented yet.\n");
      }
   }
   
   if (shall_wipe) {
      shall_write = 0;
      shall_gettag = 0;
   }
   
   if (optind==argc) {
      printhelp();
      return(0);
   }

   for (fileno=optind;fileno!=argc;fileno++) {

      file=new Mpfile;
      
      if (!(file->open(argv[fileno],(shall_write || shall_wipe)))) {
	 fprintf(stderr,"Error: Could not open file %s.\n",argv[fileno]);
         errorno=1;
      } else if (!(file->initfile(shall_gettag))) {
	 fprintf(stderr,"Error: no SYNC in file %s.\n",argv[fileno]);
         errorno=2;
      } else {
	 if (shall_write) {
            file->tag->copy(newinfo);
	    if (!file->writetag()) {
	       fprintf(stderr,"Error: Writing TAG failed on file %s.\n",argv[fileno]);
	       errorno=3;
	    }
	 }
	 
	 if (shall_wipe)
           if (!file->wipetag()) {
	     fprintf(stderr,"Error: Wiping TAG failed on file %s.\n",argv[fileno]);
	     errorno=4;
	 }
	 
	 if (shall_std) {
           if (file->mpformat(output,format,STD_BUF_LEN))
             printf("%s\n",output);
         }

	 if (shall_rename) {
	    if (!file->frename(rename)) {
	       fprintf(stderr,"Error: Renaming %s failed.\n",argv[fileno]);
	       errorno=5;
	    }
	 }
#ifdef PLAYER
	 if (shall_play) file->play();
#endif
      }
      delete(file); 
   }
   return(errorno);
}
